package com.ruoyi.system.service.impl;

import org.slf4j.LoggerFactory;
import com.ruoyi.system.mq.ServerTaskSender;
import org.springframework.scheduling.annotation.Async;
import java.util.Date;
import java.util.concurrent.CompletableFuture;
import com.ruoyi.common.core.constant.Constants;
import com.ruoyi.system.utils.SSLExpirationChecker;
import com.ruoyi.system.domain.SnSslLog;
import com.ruoyi.common.core.domain.R;
import java.util.Iterator;
import com.ruoyi.system.domain.SnServerTemplate;
import com.ruoyi.system.domain.SnServer;
import com.ruoyi.system.api.domain.SysFile;
import com.alibaba.fastjson2.JSONWriter;
import com.ruoyi.common.core.domain.StrFile;
import com.ruoyi.system.utils.jsch.AnsiStripper;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.system.utils.dbsync.runner.Syncer;
import com.ruoyi.system.enums.TaskTypeEnum;
import com.ruoyi.system.utils.jsch.ServiceShell;
import java.util.Comparator;
import java.util.Objects;
import com.ruoyi.common.core.utils.DateUtils;
import com.ruoyi.system.api.model.LoginUser;
import com.ruoyi.common.core.utils.ServletUtils;
import java.util.List;
import com.ruoyi.common.core.utils.bean.BeanUtils;
import com.ruoyi.system.domain.dto.SnServerLogDto;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.system.domain.SnServerLog;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.system.domain.SnServerTask;
import org.springframework.beans.factory.annotation.Value;
import com.ruoyi.system.service.ISnServerService;
import com.ruoyi.system.kernel.SslCountLimit;
import com.ruoyi.system.api.RemoteFileService;
import com.ruoyi.system.kernel.ServerLogKernel;
import com.ruoyi.system.mapper.SnSslLogMapper;
import com.ruoyi.system.mapper.SnServerTemplateMapper;
import com.ruoyi.system.mapper.SnServerMapper;
import com.ruoyi.system.mapper.SnServerLogMapper;
import com.ruoyi.common.security.service.TokenService;
import org.springframework.beans.factory.annotation.Autowired;
import com.ruoyi.system.mapper.SnServerTaskMapper;
import org.slf4j.Logger;
import org.springframework.stereotype.Service;
import com.ruoyi.system.service.ISnServerTaskService;

@Service
public class SnServerTaskServiceImpl implements ISnServerTaskService
{
    private static final Logger log;
    @Autowired
    private SnServerTaskMapper snServerTaskMapper;
    @Autowired
    private TokenService tokenService;
    @Autowired
    private SnServerLogMapper snServerLogMapper;
    @Autowired
    private SnServerMapper snServerMapper;
    @Autowired
    private SnServerTemplateMapper snServerTemplateMapper;
    @Autowired
    private SnSslLogMapper snSslLogMapper;
    @Autowired
    private ServerLogKernel serverLogKernel;
    @Autowired
    private RemoteFileService remoteFileService;
    @Autowired
    private SslCountLimit sslCountLimit;
    @Autowired
    private ISnServerService snServerService;
    @Value("${dbSync.url}")
    public String dbSyncUrl;
    @Value("${dbSync.password}")
    public String dbSyncPsd;
    
    public AjaxResult queryLastStatus(final SnServerTask param) {
        SnServerTask task = null;
        if (param.getServerId() != null) {
            task = this.snServerTaskMapper.queryLastTask(param.getServerId());
        }
        else if (param.getTaskId() != null) {
            task = this.snServerTaskMapper.selectSnServerTaskByTaskId(param.getTaskId());
        }
        if (task == null) {
            return AjaxResult.error("未查询到数据");
        }
        final SnServerLog log = new SnServerLog();
        log.setTaskId(task.getTaskId());
        final List<SnServerLog> logs = this.snServerLogMapper.selectSnServerLogList(log);
        return AjaxResult.success((Object)new JSONObject().fluentPut("status", (Object)task.getStatus()).fluentPut("result", (Object)task.getResult()).fluentPut("logs", (Object)BeanUtils.copyList((List)logs, (Class)SnServerLogDto.class)));
    }
    
    public SnServerTask selectSnServerTaskByTaskId(final Long taskId) {
        final SnServerTask obj = this.snServerTaskMapper.selectSnServerTaskByTaskId(taskId);
        if (obj == null || obj.getIsDelete().equals(1)) {
            return null;
        }
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        if (loginUser.getRoles().contains("admin") || loginUser.getRoles().contains("super")) {
            return obj;
        }
        if (loginUser.getRoles().contains("dealer")) {
            return obj.getDealerId().equals(loginUser.getUserid()) ? obj : null;
        }
        if (loginUser.getRoles().contains("agent")) {
            return obj.getAgentId().equals(loginUser.getUserid()) ? obj : null;
        }
        return obj.getCreateId().equals(loginUser.getUserid()) ? obj : null;
    }
    
    public List<SnServerTask> selectSnServerTaskList(final SnServerTask obj) {
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        if (loginUser.getRoles().contains("dealer")) {
            obj.setDealerId(loginUser.getUserid());
        }
        if (loginUser.getRoles().contains("agent")) {
            obj.setAgentId(loginUser.getUserid());
        }
        if (loginUser.getRoles().contains("common")) {
            obj.setCreateId(loginUser.getUserid());
        }
        obj.setIsDelete(Integer.valueOf(0));
        return this.snServerTaskMapper.selectSnServerTaskList(obj);
    }
    
    public AjaxResult insertSnServerTask(final SnServerTask snServerTask) {
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        if (!loginUser.getRoles().contains("admin") && !loginUser.getRoles().contains("super") && !loginUser.getSysUser().getUserType().equals("common")) {
            return AjaxResult.error("暂无权限");
        }
        snServerTask.setCreateId(loginUser.getUserid());
        snServerTask.setCreateName(loginUser.getUsername());
        snServerTask.setAgentId(Long.valueOf(Long.parseLong(loginUser.getSysUser().getCreateBy())));
        snServerTask.setAgentName(loginUser.getSysUser().getCreateName());
        snServerTask.setIsDelete(Integer.valueOf(0));
        snServerTask.setCreateTime(DateUtils.getNowDate());
        snServerTask.setUpdateTime(DateUtils.getNowDate());
        return AjaxResult.success((Object)this.snServerTaskMapper.insertSnServerTask(snServerTask));
    }
    
    public int updateSnServerTask(final SnServerTask snServerTask) {
        if (this.selectSnServerTaskByTaskId(snServerTask.getTaskId()) == null) {
            return 0;
        }
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        snServerTask.setUpdateId(loginUser.getUserid());
        snServerTask.setUpdateName(loginUser.getUsername());
        snServerTask.setUpdateTime(DateUtils.getNowDate());
        return this.snServerTaskMapper.updateSnServerTask(snServerTask);
    }
    
    public int deleteSnServerTaskByTaskIds(final Long[] taskIds) {
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        Long createId = null;
        if (!loginUser.getRoles().contains("admin") && !loginUser.getRoles().contains("super")) {
            createId = loginUser.getUserid();
            return this.snServerTaskMapper.deleteSnServerTaskByTaskIdsV2(taskIds, createId);
        }
        return this.snServerTaskMapper.deleteSnServerTaskByTaskIds(taskIds);
    }
    
    public int deleteSnServerTaskByTaskId(final Long taskId) {
        return this.snServerTaskMapper.deleteSnServerTaskByTaskId(taskId);
    }
    
    public void handleDoTask(final Long taskId) {
        final SnServerTask task = this.snServerTaskMapper.selectSnServerTaskByTaskId(taskId);
        if (task == null || Objects.equals(task.getIsDelete(), 1)) {
            SnServerTaskServiceImpl.log.info("未查询到任务" + taskId);
            return;
        }
        final SnServerLog param = new SnServerLog();
        param.setTaskId(taskId);
        final List<SnServerLog> logs = this.snServerLogMapper.selectSnServerLogList(param);
        if (logs.isEmpty()) {
            final SnServerTask up = new SnServerTask();
            up.setTaskId(taskId);
            up.setStatus(Integer.valueOf(3));
            up.setResult("当前任务没有需要执行的命令");
            this.snServerTaskMapper.updateSnServerTask(up);
            return;
        }
        logs.sort(Comparator.comparingLong(SnServerLog::getLogId));
        final SnServer server = this.snServerMapper.selectSnServerByServerId(task.getServerId());
        if (Objects.equals(server.getIsDelete(), 1)) {
            return;
        }
        final ServiceShell shell = new ServiceShell(task.getServerId(), this.serverLogKernel);
        try {
            if (TaskTypeEnum.isConnect(task.getType())) {
                shell.connect(server.getIp(), server.getUserName(), server.getDePassword(), (int)server.getPort());
            }
            final SnServerTask up2 = new SnServerTask();
            up2.setTaskId(task.getTaskId());
            up2.setStatus(Integer.valueOf(1));
            up2.setUpdateTime(DateUtils.getNowDate());
            this.snServerTaskMapper.updateSnServerTask(up2);
            for (final SnServerLog log : logs) {
                final long time = System.currentTimeMillis();
                final SnServerLog upObj = new SnServerLog();
                upObj.setLogId(log.getLogId());
                upObj.setStatus(Integer.valueOf(1));
                upObj.setUpdateTime(DateUtils.getNowDate());
                this.snServerLogMapper.updateSnServerLog(upObj);
                String returnLog;
                if (log.getOptName().equals("基础表结构同步")) {
                    final String targetHost = server.getIp() + ":3306";
                    upObj.setCommand(this.dbSyncUrl + "====>" + targetHost);
                    returnLog = Syncer.doSync(this.dbSyncUrl, this.dbSyncPsd, targetHost, server.getDeDbPsd(), server.getVersion());
                    if (returnLog.contains("数据库连接失败")) {
                        upObj.setLogText(returnLog);
                        upObj.setStatus(Integer.valueOf(3));
                        upObj.setTime(Long.valueOf(System.currentTimeMillis() - time));
                        upObj.setUpdateTime(DateUtils.getNowDate());
                        this.snServerLogMapper.updateSnServerLog(upObj);
                        if (task.getTaskName().equals("更新数据库表结构")) {
                            up2.setStatus(Integer.valueOf(3));
                            up2.setResult("客户端数据库连接失败，请检查3306端口是否设置白名单");
                            up2.setUpdateTime(DateUtils.getNowDate());
                            this.snServerTaskMapper.updateSnServerTask(up2);
                            this.dealUpdateServer(task);
                            return;
                        }
                        break;
                    }
                }
                else if (log.getOptName().equals("更新爬虫UA库版本")) {
                    final Boolean updateUa = Objects.equals(task.getType(), TaskTypeEnum.uaUpdate.getType());
                    returnLog = this.snServerService.syncUa(server, updateUa);
                    if (StringUtils.isNotEmpty(returnLog)) {
                        upObj.setLogText(returnLog);
                        upObj.setStatus(Integer.valueOf(3));
                        upObj.setTime(Long.valueOf(System.currentTimeMillis() - time));
                        upObj.setUpdateTime(DateUtils.getNowDate());
                        this.snServerLogMapper.updateSnServerLog(upObj);
                        if (updateUa) {
                            up2.setStatus(Integer.valueOf(3));
                            up2.setResult("执行失败");
                            up2.setUpdateTime(DateUtils.getNowDate());
                            this.snServerTaskMapper.updateSnServerTask(up2);
                            this.dealUpdateServer(task);
                            return;
                        }
                        break;
                    }
                }
                else if (log.getOptName().equals("缓存证书文件")) {
                    this.cacheEncrypt(server, task, log.getCommand());
                    returnLog = "缓存成功";
                }
                else {
                    final String command = log.getCommand();
                    if (command.startsWith("exec ")) {
                        returnLog = shell.executeCommand(command.substring(5));
                    }
                    else {
                        Integer timoutMinute = null;
                        if (Objects.equals(log.getOptName(), "安装服务器基础环境")) {
                            timoutMinute = 10;
                        }
                        returnLog = shell.sendCommand(command, timoutMinute);
                        returnLog = AnsiStripper.stripAnsi(returnLog);
                    }
                    if (log.getOptName().equals("插入商户许可证")) {
                        returnLog = this.retryInsertLicense(command, shell, returnLog, 1);
                    }
                }
                if (StringUtils.isNotEmpty(returnLog)) {
                    final StrFile strFile = new StrFile();
                    strFile.setContent(returnLog);
                    strFile.setFileName("logs_" + log.getLogId() + ".txt");
                    strFile.setContentType("text/plain");
                    final R<SysFile> r = (R<SysFile>)this.remoteFileService.uploadStrToMinioV2(JSONObject.toJSONString((Object)strFile, new JSONWriter.Feature[0]));
                    upObj.setLogText((r.getData() != null) ? ((SysFile)r.getData()).getUrl() : "");
                }
                if (this.checkError(task, upObj, time, returnLog)) {
                    return;
                }
                upObj.setStatus(Integer.valueOf(2));
                upObj.setTime(Long.valueOf(System.currentTimeMillis() - time));
                upObj.setUpdateTime(DateUtils.getNowDate());
                this.snServerLogMapper.updateSnServerLog(upObj);
            }
            up2.setStatus(Integer.valueOf(2));
            up2.setResult("执行完成");
            up2.setUpdateTime(DateUtils.getNowDate());
            this.snServerTaskMapper.updateSnServerTask(up2);
            if (Objects.equals(task.getType(), TaskTypeEnum.installBase.getType())) {
                final SnServer serUp = new SnServer();
                serUp.setServerId(task.getServerId());
                serUp.setStatus(Integer.valueOf(2));
                serUp.setUpdateTime(DateUtils.getNowDate());
                this.snServerMapper.updateSnServer(serUp);
            }
            if (Objects.equals(task.getType(), TaskTypeEnum.installClient.getType())) {
                final SnServerTemplate seTempUp = new SnServerTemplate();
                seTempUp.setId(task.getServerTemplateId());
                seTempUp.setStatus(Integer.valueOf(2));
                seTempUp.setUpdateTime(DateUtils.getNowDate());
                this.snServerTemplateMapper.updateSnServerTemplate(seTempUp);
            }
            if (task.getProcess() != null) {
                final SnServer serUp = new SnServer();
                serUp.setServerId(task.getServerId());
                serUp.setProcess(task.getProcess());
                serUp.setTaskId(Long.valueOf(0L));
                serUp.setUpdateTime(DateUtils.getNowDate());
                this.snServerMapper.updateSnServer(serUp);
            }
        }
        catch (final Exception e) {
            final SnServerTask up3 = new SnServerTask();
            up3.setTaskId(task.getTaskId());
            up3.setStatus(Integer.valueOf(3));
            up3.setResult("任务执行异常：" + e.getMessage());
            up3.setUpdateTime(DateUtils.getNowDate());
            this.snServerTaskMapper.updateSnServerTask(up3);
            this.dealUpdateServer(task);
        }
        finally {
            shell.disconnect();
        }
    }
    
    private Boolean checkError(final SnServerTask task, final SnServerLog upObj, final long startTime, final String returnLog) {
        String msg = "";
        if (StringUtils.isEmpty(returnLog)) {
            return Boolean.FALSE;
        }
        if (Objects.equals(task.getType(), TaskTypeEnum.deploySSL.getType())) {
            if (returnLog.contains("domains in the last 168 hours")) {
                msg = "部署次数超限";
            }
            else if (returnLog.contains("DNS A/AAAA record(s) for that domain")) {
                msg = "域名未解析到当前IP";
            }
            else if (returnLog.contains("Which services should be restarted")) {
                msg = "安装过程中遇到需要选择服务重启，无法自动选择，请重新安装证书";
            }
            else if (returnLog.contains("Certbot failed to authenticate some domains")) {
                msg = "未能对域名进行身份验证，请确保域名正确解析到IP";
            }
        }
        if (returnLog.contains("空闲弹出")) {
            msg = "执行异常，超时关闭通道";
        }
        if (StringUtils.isNotEmpty(msg)) {
            upObj.setStatus(Integer.valueOf(3));
            upObj.setTime(Long.valueOf(System.currentTimeMillis() - startTime));
            upObj.setUpdateTime(DateUtils.getNowDate());
            this.snServerLogMapper.updateSnServerLog(upObj);
            final SnServerTask up = new SnServerTask();
            up.setTaskId(task.getTaskId());
            up.setStatus(Integer.valueOf(3));
            up.setResult(msg);
            up.setUpdateTime(DateUtils.getNowDate());
            this.snServerTaskMapper.updateSnServerTask(up);
            return Boolean.TRUE;
        }
        return Boolean.FALSE;
    }
    
    private void dealUpdateServer(final SnServerTask task) {
        SnServer serUp = null;
        if (Objects.equals(task.getType(), TaskTypeEnum.installBase.getType())) {
            serUp = new SnServer();
            serUp.setServerId(task.getServerId());
            serUp.setStatus(Integer.valueOf(3));
            serUp.setDomain("");
            serUp.setVersion("");
            serUp.setBackVersion("");
            serUp.setUiVersion("");
            serUp.setDbPsd("");
        }
        if (Objects.equals(task.getType(), TaskTypeEnum.updateBaseUI.getType())) {
            serUp = new SnServer();
            serUp.setServerId(task.getServerId());
            serUp.setVersion(task.getClientVersion());
            serUp.setUiVersion(task.getBizVersion());
        }
        if (Objects.equals(task.getType(), TaskTypeEnum.updateClient.getType())) {
            final SnServerTemplate seTempUp = new SnServerTemplate();
            seTempUp.setId(task.getServerTemplateId());
            seTempUp.setVersion(task.getBizVersion());
            this.snServerTemplateMapper.updateSnServerTemplate(seTempUp);
        }
        if (Objects.equals(task.getType(), TaskTypeEnum.updateBack.getType())) {
            serUp = new SnServer();
            serUp.setServerId(task.getServerId());
            serUp.setVersion(task.getClientVersion());
            serUp.setBackVersion(task.getBizVersion());
        }
        if (Objects.equals(task.getType(), TaskTypeEnum.uaUpdate.getType())) {
            serUp = new SnServer();
            serUp.setServerId(task.getServerId());
            serUp.setUaVersion(task.getBizVersion());
        }
        if (serUp != null) {
            this.snServerMapper.updateSnServer(serUp);
        }
    }
    
    private String retryInsertLicense(final String command, final ServiceShell shell, String returnLog, int num) {
        try {
            if (returnLog.contains("Can't connect to local MySQL")) {
                if (num > 5) {
                    return "重试5次未能成功连接数据库";
                }
                Thread.sleep(2000L);
                if (command.startsWith("exec ")) {
                    returnLog = shell.executeCommand(command.substring(5));
                }
                else {
                    returnLog = shell.sendCommand(command);
                    returnLog = AnsiStripper.stripAnsi(returnLog);
                }
                ++num;
                this.retryInsertLicense(command, shell, returnLog, num);
            }
        }
        catch (final Exception ex) {}
        return returnLog;
    }
    
    @Async
    public void cacheEncrypt(final SnServer server, final SnServerTask task, final String command) {
        final ServiceShell shell = new ServiceShell(server.getServerId(), this.serverLogKernel);
        final SnSslLog sslLog = new SnSslLog();
        sslLog.setExpireTime(DateUtils.currentTimeAdd(90));
        try {
            final String[] arr = command.split(" && ");
            final String pemCommand = arr[0];
            final String keyCommand = arr[1];
            final String domain = pemCommand.split("/")[4];
            sslLog.setDomain(domain);
            this.sslCountLimit.addCount(domain, server.getIp());
            shell.connect(server.getIp(), server.getUserName(), server.getDePassword(), (int)server.getPort());
            final String pemStr = shell.executeCommand(pemCommand);
            sslLog.setExpireTime(SSLExpirationChecker.getCertificateExDate(pemStr));
            final String keyStr = shell.executeCommand(keyCommand);
            StrFile strFile = new StrFile();
            strFile.setContent(pemStr);
            strFile.setFileName("fullchain.pem");
            strFile.setContentType("text/plain");
            R<SysFile> r = (R<SysFile>)this.remoteFileService.uploadStrToMinio(strFile);
            if (r.getCode() != Constants.SUCCESS) {
                CompletableFuture.completedFuture((Object)null);
                return;
            }
            final String pemUrl = ((SysFile)r.getData()).getUrl();
            sslLog.setPem(pemUrl);
            strFile = new StrFile();
            strFile.setContent(keyStr);
            strFile.setFileName("privkey.pem");
            strFile.setContentType("text/plain");
            r = (R<SysFile>)this.remoteFileService.uploadStrToMinio(strFile);
            if (r.getCode() != Constants.SUCCESS) {
                CompletableFuture.completedFuture((Object)null);
                return;
            }
            final String keyUrl = ((SysFile)r.getData()).getUrl();
            sslLog.setKey(keyUrl);
            sslLog.setCreateId(sslLog.getCreateId());
            sslLog.setCreateName(sslLog.getCreateName());
            sslLog.setCreateTime(new Date());
            this.snSslLogMapper.insertSnSslLog(sslLog);
        }
        catch (final Exception e) {
            SnServerTaskServiceImpl.log.error("缓存证书失败：" + e.getMessage());
        }
        finally {
            shell.disconnect();
        }
        if (task.getTaskName().equals("部署基础环境ssl证书")) {
            final SnServer up = new SnServer();
            up.setServerId(task.getServerId());
            up.setSslTime(sslLog.getExpireTime());
            up.setSslPem(sslLog.getPem());
            up.setSslKey(sslLog.getKey());
            this.snServerMapper.updateSnServer(up);
        }
        else if (task.getTaskName().startsWith("部署客户端域名ssl证书")) {
            final SnServerTemplate up2 = new SnServerTemplate();
            up2.setId(task.getServerTemplateId());
            up2.setSslTime(sslLog.getExpireTime());
            up2.setSslPem(sslLog.getPem());
            up2.setSslKey(sslLog.getKey());
            this.snServerTemplateMapper.updateSnServerTemplate(up2);
        }
        CompletableFuture.completedFuture((Object)null);
    }
    
    static {
        log = LoggerFactory.getLogger(ServerTaskSender.class);
    }
}
